/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.customer.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.customer.entity.BizCustomerAccount;
import vip.xiaonuo.biz.modular.customer.param.BizCustomerAccountAddParam;
import vip.xiaonuo.biz.modular.customer.param.BizCustomerAccountIdParam;
import vip.xiaonuo.biz.modular.customer.param.BizCustomerAccountPageParam;
import vip.xiaonuo.biz.modular.customer.service.BizCustomerAccountService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 客户账号控制器
 *
 * @author sandy
 * @date  2025/04/09 17:00
 */
@Tag(name = "客户账号控制器")
@RestController
@Validated
public class BizCustomerAccountController {

    @Resource
    private BizCustomerAccountService bizCustomerAccountService;

    /**
     * 获取客户账号分页
     *
     * @author sandy
     * @date  2025/04/09 17:00
     */
    @Operation(summary = "获取客户账号分页")
    @GetMapping("/biz/customerAccount/page")
    public CommonResult<Page<BizCustomerAccount>> page(BizCustomerAccountPageParam bizCustomerAccountPageParam) {
        return CommonResult.data(bizCustomerAccountService.page(bizCustomerAccountPageParam));
    }

    /**
     * 添加客户账号
     *
     * @author sandy
     * @date  2025/04/09 17:00
     */
    @Operation(summary = "添加客户账号")
    @CommonLog("添加客户账号")
    @SaCheckPermission("/biz/customerAccount/add")
    @PostMapping("/biz/customerAccount/add")
    public CommonResult<String> add(@RequestBody @Valid BizCustomerAccountAddParam bizCustomerAccountAddParam) {
        bizCustomerAccountService.add(bizCustomerAccountAddParam);
        return CommonResult.ok();
    }

    /**
     * 删除客户账号
     *
     * @author sandy
     * @date  2025/04/09 17:00
     */
    @Operation(summary = "删除客户账号")
    @CommonLog("删除客户账号")
    @SaCheckPermission("/biz/customerAccount/delete")
    @PostMapping("/biz/customerAccount/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizCustomerAccountIdParam> bizCustomerAccountIdParamList) {
        bizCustomerAccountService.delete(bizCustomerAccountIdParamList);
        return CommonResult.ok();
    }

}
