/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.receiptcompany.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.receiptcompany.entity.BizReceiptCompany;
import vip.xiaonuo.biz.modular.receiptcompany.param.BizReceiptCompanyAddParam;
import vip.xiaonuo.biz.modular.receiptcompany.param.BizReceiptCompanyEditParam;
import vip.xiaonuo.biz.modular.receiptcompany.param.BizReceiptCompanyIdParam;
import vip.xiaonuo.biz.modular.receiptcompany.param.BizReceiptCompanyPageParam;
import vip.xiaonuo.biz.modular.receiptcompany.service.BizReceiptCompanyService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 收货单位控制器
 *
 * @author 徐超
 * @date  2025/01/02 17:45
 */
@Tag(name = "收货单位控制器")
@RestController
@Validated
public class BizReceiptCompanyController {

    @Resource
    private BizReceiptCompanyService bizReceiptCompanyService;

    /**
     * 获取收货单位分页
     *
     * @author 徐超
     * @date  2025/01/02 17:45
     */
    @Operation(summary = "获取收货单位分页")
    @SaCheckPermission("/biz/receiptcompany/page")
    @GetMapping("/biz/receiptcompany/page")
    public CommonResult<Page<BizReceiptCompany>> page(BizReceiptCompanyPageParam bizReceiptCompanyPageParam) {
        return CommonResult.data(bizReceiptCompanyService.page(bizReceiptCompanyPageParam));
    }

    /**
     * 添加收货单位
     *
     * @author 徐超
     * @date  2025/01/02 17:45
     */
    @Operation(summary = "添加收货单位")
    @CommonLog("添加收货单位")
    @SaCheckPermission("/biz/receiptcompany/add")
    @PostMapping("/biz/receiptcompany/add")
    public CommonResult<String> add(@RequestBody @Valid BizReceiptCompanyAddParam bizReceiptCompanyAddParam) {
        bizReceiptCompanyService.add(bizReceiptCompanyAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑收货单位
     *
     * @author 徐超
     * @date  2025/01/02 17:45
     */
    @Operation(summary = "编辑收货单位")
    @CommonLog("编辑收货单位")
    @SaCheckPermission("/biz/receiptcompany/edit")
    @PostMapping("/biz/receiptcompany/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizReceiptCompanyEditParam bizReceiptCompanyEditParam) {
        bizReceiptCompanyService.edit(bizReceiptCompanyEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除收货单位
     *
     * @author 徐超
     * @date  2025/01/02 17:45
     */
    @Operation(summary = "删除收货单位")
    @CommonLog("删除收货单位")
    @SaCheckPermission("/biz/receiptcompany/delete")
    @PostMapping("/biz/receiptcompany/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizReceiptCompanyIdParam> bizReceiptCompanyIdParamList) {
        bizReceiptCompanyService.delete(bizReceiptCompanyIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取收货单位详情
     *
     * @author 徐超
     * @date  2025/01/02 17:45
     */
    @Operation(summary = "获取收货单位详情")
    @SaCheckPermission("/biz/receiptcompany/detail")
    @GetMapping("/biz/receiptcompany/detail")
    public CommonResult<BizReceiptCompany> detail(@Valid BizReceiptCompanyIdParam bizReceiptCompanyIdParam) {
        return CommonResult.data(bizReceiptCompanyService.detail(bizReceiptCompanyIdParam));
    }
}
